@@ -61,7 +61,7 @@ class tdcoa:
     systemspath = ''
     filesetpath = ''
     outputpath = ''
-    version = "0.3.9.6.6"
+    version = "0.3.9.6.9"
     skip_dbs = False
 
     # dictionaries
@@ -98,6 +98,34 @@ class tdcoa:
         # filesets.yaml is validated at download time
 
         self.reload_config()
+        if os.path.exists(self.filesetpath) and os.path.exists(self.systemspath):
+            self.update_sourcesystem_yaml()
+
+    # Function to add Source system yaml file with all the fileset entries in the filesets.yaml file and set active to false
+    def update_sourcesystem_yaml(self):
+        with open(self.systemspath, 'r') as fh:
+            filesetstr = fh.read()
+        sourcesysyaml = yaml.load(filesetstr, Loader=yaml.FullLoader)
+        fh.close()
+        with open(self.filesetpath, 'r') as fh:
+            filesetstr = fh.read()
+        filesetsyaml = yaml.load(filesetstr, Loader=yaml.FullLoader)
+        fh.close()
+        filesets = []
+        for fileset, filesetobj in filesetsyaml.items():
+            filesets.append(fileset)
+
+        for sysname in sourcesysyaml['systems'].keys():
+            sys_filesets = list(sourcesysyaml['systems'][sysname]['filesets'].keys())
+            missing_filesets = list(set(filesets) - set(sys_filesets))
+            for fileset in missing_filesets:
+                sourcesysyaml['systems'][sysname]['filesets'][fileset] = {}
+                sourcesysyaml['systems'][sysname]['filesets'][fileset]['active'] = 'False'
+
+        with open(self.systemspath, 'w') as fh:
+            fh.write(yaml.dump(sourcesysyaml))
+        fh.close()
+        self.utils.log('Updated the Sourcesystem yaml file with all the fileset entries in the filesets.yaml file')
 
     def reload_config(self, configpath='', secretpath='', systemspath='', refresh_defaults=False):
         """Reloads configuration YAML files (config & secrets) used as
@@ -146,77 +174,53 @@ class tdcoa:
         self.utils.log('time', str(dt.datetime.now()))
         self.utils.log('tdcoa version', self.version)
 
+
         # ensure all required configuration files are present:
         self.utils.log('checking core config files')
-        startfiles = ['coa.py','secrets.yaml','config.yaml','source_systems.yaml']
+        startfiles = ['secrets.yaml','config.yaml','source_systems.yaml','run_gui.py','run_cmdline.py','run_cmdline']
         startfilecontent = ''
         for startfile in startfiles:
             startfile_src = os.path.join(os.path.dirname(tdcsm.__file__), startfile)
+            startfile_ovr = os.path.join(self.approot,'0_override', startfile)
             startfile_dst = os.path.join(self.approot, startfile)
+
+            # remove files if "refresh defaults" is requested via __init__ param
             if self.refresh_defaults and os.path.isfile(startfile_dst) and startfiles != 'secrets.yaml':
                 os.remove(startfile_dst)
+
+            # if file is missing:
             if not os.path.isfile(startfile_dst):
-                self.utils.log(' MISSING FILE, REPLACING WITH TEMPLATE ', startfile)
-                if os.path.isfile(startfile_src):
+                self.utils.log(' MISSING FILE', startfile)
+                # check if the file is in the 0_override folder... if so, use that:
+                if os.path.isfile(startfile_ovr):
+                    self.utils.log('   Adding from 0_override')
+                    with open(startfile_ovr) as f1:
+                        startfilecontent = f1.read()
+                # if no override, pull from package directory
+                elif os.path.isfile(startfile_src):
+                    self.utils.log('   Adding from installed package')
                     with open(startfile_src) as f1:
                         startfilecontent = f1.read()
-                        self.utils.log('  template file pulled from package')
                 else:  # this is just-in-case, until I can be sure above logic is working.
                     if startfile == 'coa.py': startfilecontent = 'from tdcsm.tdgui import coa\nc=coa()'
                     if startfile == 'secrets.yaml': startfilecontent = 'secrets:\n  td_quicklook: "qlikuserid"\n  td_password:  "qlikpassword"'
                     if startfile == 'config.yaml': startfilecontent = self.yaml_config()
                     if startfile == 'source_systems.yaml': startfilecontent = self.yaml_systems()
-                    self.utils.log('  template file generated from backup')
+                    self.utils.log('   Adding from internal string (should not happen)')
                 with open(startfile_dst, 'w') as f2:
                     f2.write(startfilecontent)
 
 
 
-        # remove 2 default files -> config, source_systems
-        # todo NOTE: currently not removing secrets file because it shouldnt require format updates
-        #if self.refresh_defaults:
-        #    self.utils.log('\nRefresh defaults = True     **Note: secrets.yaml will not be changed**')
-        #    if os.path.isfile(self.configpath):
-        #        os.remove(self.configpath)
-        #        self.utils.log('old config.yaml found and removed')
-        #    if os.path.isfile(self.systemspath):
-        #        os.remove(self.systemspath)
-        #        self.utils.log('old source_systems.yaml found and removed')
-        #    # if os.path.isfile(self.secretpath):
-        #    #     os.remove(self.secretpath)
-        #    self.utils.log('')
-
-        # load secrets. create default if missing
-        #self.utils.log('loading secrets', os.path.basename(self.secretpath))
-        #if not os.path.isfile(self.secretpath):
-        #    self.utils.log('creating default secrets', self.secretpath)
-        #    with open(os.path.join(os.path.dirname(tdcsm.__file__), 'secrets.yaml')) as f:
-        #        with open(secretpath, 'w') as f2:
-        #            f2.write(f.read())
-        #else:
-        #    self.utils.log('found secrets.yaml', self.secretpath)
-
+        # load secrets.yaml
         with open(secretpath, 'r') as fh:
             secretstr = fh.read()
         self.secrets = yaml.load(secretstr, Loader=yaml.FullLoader)['secrets']
         self.utils.secrets = self.secrets  # update secrets attribute in logger
 
-        # load config. create default if missing
-        #if self.refresh_defaults:
-        #    self.utils.log('loading config', os.path.basename(self.configpath) + ' [FORCED REFRESH]')
-        #else:
-        #    self.utils.log('loading config', os.path.basename(self.configpath))
-        #if not os.path.isfile(self.configpath) or self.refresh_defaults:
-        #    self.utils.log('creating default config', self.configpath)
-        #    with open(os.path.join(os.path.dirname(tdcsm.__file__), 'config.yaml')) as f:
-        #        with open(configpath, 'w') as f2:
-        #            f2.write(f.read())
-        #else:
-        #    self.utils.log('found config.yaml', self.configpath)
-
+        # load config.yaml
         with open(configpath, 'r') as fh:
             configstr = fh.read()
-
         configyaml = yaml.load(configstr, Loader=yaml.FullLoader)
         configstr = self.utils.substitute(configstr, self.secrets, 'secrets')
         configstr = self.utils.substitute(configstr, configyaml['substitutions'], 'config:substitutions')
@@ -228,23 +232,8 @@ class tdcoa:
         # load substitutions
         self.utils.log('loading dictionary', 'substitutions')
         self.substitutions = configyaml['substitutions']
-
-        # load source systems. create default if missing
-        #if self.refresh_defaults:
-        #    self.utils.log('loading source systems', os.path.basename(self.systemspath) + ' [FORCED REFRESH]')
-        #else:
-        #    self.utils.log('loading source systems', os.path.basename(self.systemspath))
-        #if not os.path.isfile(self.systemspath) or self.refresh_defaults:
-        #    self.utils.log('creating default source_systems', self.systemspath)
-        #    with open(os.path.join(os.path.dirname(tdcsm.__file__), 'source_systems.yaml')) as f:
-        #        with open(systemspath, 'w') as f2:
-        #            f2.write(f.read())
-        #else:
-        #    self.utils.log('found source_systems.yaml', self.systemspath)
-
         with open(systemspath, 'r') as fh:
             systemsstr = fh.read()
-
         systemsstr = self.utils.substitute(systemsstr, self.secrets, 'secrets')
         systemsstr = self.utils.substitute(systemsstr, self.substitutions, 'systems:substitutions')
         systemsyaml = yaml.load(systemsstr, Loader=yaml.FullLoader)
@@ -281,10 +270,11 @@ class tdcoa:
                                      'True'])
 
         # add skip_dbs back in as silent (unlisted) option
+        self.skip_dbs = False
         if 'skip_dbs' in self.settings:
             if self.settings['skip_dbs'].strip().lower() == 'true':
                 self.skip_dbs = True
-                
+
         self.filesetpath = self.settings['localfilesets']
 
         # create missing folders
@@ -485,7 +475,9 @@ class tdcoa:
 
                         else:  # not found
                             self.utils.log(' not found in filesets.yaml', sys_setname)
-
+        # Update the Sourcesystem yaml file with all the available filesets in the filesets.yaml
+        if os.path.exists(self.filesetpath) and os.path.exists(self.systemspath):
+            self.update_sourcesystem_yaml()
         self.utils.log('\ndone!')
         self.utils.log('time', str(dt.datetime.now()))
 
@@ -543,15 +535,6 @@ class tdcoa:
                             if dbsversion_match and collection_match:
                                 shutil.copyfile(os.path.join(srcpath, downloaded_file), os.path.join(dstpath, downloaded_file))
 
-
-
-
-
-
-
-
-
-
         self.utils.log('\ndone!')
         self.utils.log('time', str(dt.datetime.now()))
 
@@ -1021,7 +1004,10 @@ class tdcoa:
                                                 sqlcmd = self.utils.get_special_commands(sql)
                                                 sql = sqlcmd.pop('sql', '')
 
+
                                                 df = self.utils.open_sql(conn, sql, skip = self.skip_dbs)  # <--------------------- Run SQL
+                                                csvfile=''
+                                                csvfile_exists=False
 
                                                 if len(df) != 0:  # Save non-empty returns to .csv
 
@@ -1059,14 +1045,20 @@ class tdcoa:
                                                     else:  # if odbc conn type, merge in column names
                                                         df.to_csv(csvfile, header=col_names)  # <---------------------- Save to .csv
                                                     self.utils.log('file saved!')
-
+                                                    csvfile_exists = os.path.exists(csvfile)
                                                 if 'vis' in sqlcmd:  # run visualization py file
-                                                    self.utils.log('\nvis cmd', 'found')
-                                                    vis_file = os.path.join(workpath, sqlcmd['vis'].replace('.csv', '.py'))
-                                                    self.utils.log('vis py file', vis_file)
-                                                    self.utils.log('running vis file..')
-                                                    os.system('python %s' % vis_file)
-                                                    self.utils.log('Vis file complete!')
+                                                    if csvfile_exists == False:  # Avoid load error by skipping the manifest file entry if SQL returns zero records.
+                                                        self.utils.log(
+                                                            'The SQL returned Zero records and hence the file was not generated, So skipping the vis special command',
+                                                            csvfile)
+                                                    else:
+
+                                                        self.utils.log('\nvis cmd', 'found')
+                                                        vis_file = os.path.join(workpath, sqlcmd['vis'].replace('.csv', '.py'))
+                                                        self.utils.log('vis py file', vis_file)
+                                                        self.utils.log('running vis file..')
+                                                        os.system('python %s' % vis_file)
+                                                        self.utils.log('Vis file complete!')
 
                                                 if 'pptx' in sqlcmd:  # insert to pptx file
                                                     self.utils.log('\npptx cmd', 'found')
@@ -1077,19 +1069,25 @@ class tdcoa:
                                                     self.utils.log('pptx file complete!')
 
                                                 if 'load' in sqlcmd:  # add to manifest
-                                                    self.utils.log(
-                                                        'file marked for loading to Transcend, adding to upload-manifest.json')
-                                                    if 'call' not in sqlcmd:
-                                                        sqlcmd['call'] = ''
-                                                    manifest_entry = '%s{"file": "%s",  "table": "%s",  "call": "%s"}' % (
+
+                                                    if csvfile_exists == False: #Avoid load error by skipping the manifest file entry if SQL returns zero records.
+                                                        self.utils.log('The SQL returned Zero records and hence the file was not generated, So skipping the manifest entry',
+                                                                       csvfile)
+                                                    else:
+                                                        self.utils.log(
+                                                            'file marked for loading to Transcend, adding to upload-manifest.json')
+                                                        if 'call' not in sqlcmd:
+                                                            sqlcmd['call'] = ''
+
+                                                        manifest_entry = '%s{"file": "%s",  "table": "%s",  "call": "%s"}' % (
                                                         manifestdelim, sqlcmd['save'], sqlcmd['load'],
                                                         sqlcmd['call'])
-                                                    manifestdelim = '\n,'
+                                                        manifestdelim = '\n,'
 
-                                                    with open(os.path.join(outputfo, 'upload-manifest.json'),
+                                                        with open(os.path.join(outputfo, 'upload-manifest.json'),
                                                               'a') as manifest:
-                                                        manifest.write(manifest_entry)
-                                                        self.utils.log('Manifest updated',
+                                                            manifest.write(manifest_entry)
+                                                            self.utils.log('Manifest updated',
                                                                  str(manifest_entry).replace(',', ',\n'))
 
                                         # archive file we just processed (for re-run-ability)
@@ -1244,13 +1242,13 @@ class tdcoa:
                                     self.utils.log('perm table', entry['schema'] + '.' + entry['table'] + '_%s' % self.unique_id)
 
                                     # create staging table (perm) with unique id
-                                    if self.settings['skip_dbs'] != 'True':
+                                    if not self.skip_dbs:
                                         copy_to_sql(dfcsv, entry['table'] + '_%s' % self.unique_id, entry['schema'], if_exists='replace')
                                     self.utils.log('complete', str(dt.datetime.now()))
 
                                     # load to GTT
                                     self.utils.log('\nload to GTT', entry['schema'] + '.' + entry['table'])
-                                    if self.settings['skip_dbs'] != 'True':
+                                    if not self.skip_dbs:
                                         transcend['connection'].execute("""
                                         INSERT INTO {db}.{table} SELECT * FROM {db}.{unique_table}
                                         """.format(db=entry['schema'],
@@ -1266,7 +1264,7 @@ class tdcoa:
                                 # 2. call sp on GTT to merge to final table
                                 else:
                                     self.utils.log('write_to_perm', 'False')
-                                    if self.settings['skip_dbs'] != 'True':
+                                    if not self.skip_dbs:
                                         copy_to_sql(dfcsv, entry['table'], entry['schema'], if_exists='append')
                                     self.utils.log('complete', str(dt.datetime.now()))
                                     successful_load = True
@@ -1295,7 +1293,7 @@ class tdcoa:
                             if str(entry['call']).strip() != "" and successful_load:
                                 self.utils.log('\nStored Proc', str(entry['call']))
                                 try:
-                                    if self.settings['skip_dbs'] != 'True':
+                                    if not self.skip_dbs:
                                         transcend['connection'].execute('call %s ;' % str(entry['call']))
                                     self.utils.log('complete', str(dt.datetime.now()))
 
@@ -1303,7 +1301,7 @@ class tdcoa:
                                     if self.settings['write_to_perm'].lower() == 'true':
                                         self.utils.log('\ndrop unique perm table', entry['schema'] + '.' + entry['table'] + '_%s' % self.unique_id)
 
-                                        if self.settings['skip_dbs'] != 'True':
+                                        if not self.skip_dbs:
                                             transcend['connection'].execute("""
                                             DROP TABLE {db}.{unique_table}
                                             """.format(db=entry['schema'],
